from test import *

class DeviceTest(TestCase):
    def test_devices(self):
        for device in NetworkManager.NetworkManager.Devices:
            self.assertIsStrictSubclass(type(device), NetworkManager.Device)
            if device.Dhcp4Config:
                self.assertIsInstance(device.Dhcp4Config, NetworkManager.DHCP4Config)
            if device.Dhcp6Config:
                self.assertIsInstance(device.Dhcp6Config, NetworkManager.DHCP6Config)
            if device.Ip4Config:
                self.assertIsInstance(device.Ip4Config, NetworkManager.IP4Config)
            if device.Ip6Config:
                self.assertIsInstance(device.Ip6Config, NetworkManager.IP6Config)
            if device.Ip4Address:
                self.assertIsIpAddress(device.Ip4Address)
            if hasattr(device, 'HwAddress') and device.HwAddress:
                self.assertIsMacAddress(device.HwAddress)
            if hasattr(device, 'PermHwAddress') and device.PermHwAddress:
                self.assertIsMacAddress(device.PermHwAddress)
        if device.DeviceType == NetworkManager.NM_DEVICE_TYPE_WIFI:
            for ap in device.AccessPoints:
                self.assertIsInstance(ap, NetworkManager.AccessPoint)
            device.RequestScan({})
        elif device.DeviceType == NetworkManager.NM_DEVICE_TYPE_ETHERNET:
            self.assertIn(device.Carrier, (True, False))
        elif device.DeviceType == NetworkManager.NM_DEVICE_TYPE_GENERIC:
            self.assertIsInstance(device.TypeDescription, six.text_type)
        elif device.DeviceType == NetworkManager.NM_DEVICE_TYPE_TUN:
            if device.Owner != -1:
                import pwd
                pwd.getpwuid(device.Owner)
        else:
            self.fail("I don't know how to test %s devices" % type(device).__name__)

if __name__ == '__main__':
    unittest.main()



from test import *

class NetworkManagerTest(TestCase):
    def test_properties(self):
        self.assertIsInstance(NetworkManager.NetworkManager.NetworkingEnabled, bool)
        self.assertIsInstance(NetworkManager.NetworkManager.Metered, int)
        self.assertIsInstance(NetworkManager.NetworkManager.Version, six.string_types)
        self.assertIsInstance(NetworkManager.NetworkManager.ActiveConnections, list)
        for conn in NetworkManager.NetworkManager.ActiveConnections:
            self.assertIsInstance(conn, NetworkManager.ActiveConnection)
        self.assertIsInstance(NetworkManager.NetworkManager.Devices, list)
        for dev in NetworkManager.NetworkManager.Devices:
            self.assertIsInstance(dev, NetworkManager.Device)
        self.assertIsInstance(NetworkManager.NetworkManager.PrimaryConnection, NetworkManager.ActiveConnection)

    @unittest.skipUnless(have_permission('sleep-wake'), "Not allowed to make networkmanager sleep")
    def test_sleep(self):
        NetworkManager.NetworkManager.Sleep(True)
        self.assertRaisesDBus('AlreadyAsleepOrAwake', NetworkManager.NetworkManager.Sleep, True)
        NetworkManager.NetworkManager.Sleep(False)
        self.assertRaisesDBus('AlreadyAsleepOrAwake', NetworkManager.NetworkManager.Sleep, False)
        self.waitForConnection()

    def test_enable(self):
        NetworkManager.NetworkManager.Enable(False)
        self.assertRaisesDBus('AlreadyEnabledOrDisabled', NetworkManager.NetworkManager.Enable, False)
        NetworkManager.NetworkManager.Enable(True)
        self.assertRaisesDBus('AlreadyEnabledOrDisabled', NetworkManager.NetworkManager.Enable, True)
        self.waitForConnection()

    @unittest.skipUnless(os.getuid() == 0, "Must be root to modify logging")
    def test_logging(self):
        level1, domains = NetworkManager.NetworkManager.GetLogging()
        self.assertIn(level1, ['ERR', 'WARN', 'INFO', 'DEBUG', 'TRACE'])
        self.assertIn('PLATFORM', domains)
        NetworkManager.NetworkManager.SetLogging("KEEP", "PLATFORM:DEBUG")
        level2, domains = NetworkManager.NetworkManager.GetLogging()
        self.assertEqual(level1, level2)
        self.assertIn('PLATFORM:DEBUG', domains)
        self.assertIn('CORE', domains)
        NetworkManager.NetworkManager.SetLogging("KEEP", "PLATFORM:" + level1)
        level2, domains = NetworkManager.NetworkManager.GetLogging()
        self.assertIn('PLATFORM', domains)
        self.assertNotIn('PLATFORM:DEBUG', domains)

    def test_permissions(self):
        permissions = NetworkManager.NetworkManager.GetPermissions()
        self.assertIsInstance(permissions, dict)
        for key in permissions:
            self.assertTrue(key.startswith('org.freedesktop.NetworkManager.'))
            self.assertIn(permissions[key], ('yes', 'no', 'auth'))

    def test_devices(self):
        dev1 = NetworkManager.NetworkManager.GetDevices()
        dev2 = NetworkManager.NetworkManager.GetAllDevices()
        for dev in dev1:
            self.assertIsInstance(dev, NetworkManager.Device)
            self.assertIsStrictSubclass(dev.__class__, NetworkManager.Device)
            self.assertIn(dev, dev2)
        dev = NetworkManager.NetworkManager.GetDeviceByIpIface(dev1[0].IpInterface)
        self.assertEqual(dev, dev1[0])

if __name__ == '__main__':
    unittest.main()



"""
Display information about everything network-related that network-manager can
say something about.
"""

import NetworkManager
c = NetworkManager.const

print("%-30s %s" % ("Version:", NetworkManager.NetworkManager.Version))
print("%-30s %s" % ("Hostname:", NetworkManager.Settings.Hostname))
print("%-30s %s" % ("Can modify:", NetworkManager.Settings.CanModify))
print("%-30s %s" % ("Networking enabled:", NetworkManager.NetworkManager.NetworkingEnabled))
print("%-30s %s" % ("Wireless enabled:", NetworkManager.NetworkManager.WirelessEnabled))
print("%-30s %s" % ("Wireless hw enabled:", NetworkManager.NetworkManager.WirelessHardwareEnabled))
print("%-30s %s" % ("Wwan enabled:", NetworkManager.NetworkManager.WwanEnabled))
print("%-30s %s" % ("Wwan hw enabled:", NetworkManager.NetworkManager.WwanHardwareEnabled))
print("%-30s %s" % ("Wimax enabled:", NetworkManager.NetworkManager.WimaxEnabled))
print("%-30s %s" % ("Wimax hw enabled:", NetworkManager.NetworkManager.WimaxHardwareEnabled))
print("%-30s %s" % ("Overall state:", c('state', NetworkManager.NetworkManager.State)))

print("")

print("Permissions")
for perm, val in sorted(NetworkManager.NetworkManager.GetPermissions().items()):
    print("%-30s %s" % (perm[31:] + ':', val))

print("")

print("Available network devices")
print("%-10s %-19s %-20s %s" % ("Name", "State", "Driver", "Managed?"))
for dev in NetworkManager.NetworkManager.GetDevices():
    print("%-10s %-19s %-20s %s" % (dev.Interface, c('device_state', dev.State), dev.Driver, dev.Managed))

print("")

print("Available connections")
print("%-30s %s" % ("Name", "Type"))
for conn in NetworkManager.Settings.ListConnections():
    settings = conn.GetSettings()['connection']
    print("%-30s %s" % (settings['id'], settings['type']))

print("")

print("Active connections")
print("%-30s %-20s %-10s %s" % ("Name", "Type", "Default", "Devices"))
for conn in NetworkManager.NetworkManager.ActiveConnections:
    settings = conn.Connection.GetSettings()['connection']
    print("%-30s %-20s %-10s %s" % (settings['id'], settings['type'], conn.Default, ", ".join([x.Interface for x in conn.Devices])))



from test import *

class ActiveConnectionTest(TestCase):
    def test_properties(self):
        for conn in NetworkManager.NetworkManager.ActiveConnections:
            self.assertIsInstance(conn.Connection, NetworkManager.Connection)
            for device in conn.Devices:
                self.assertIsInstance(device, NetworkManager.Device)
            if conn.Connection.GetSettings()['connection']['type'] == '802-11-wireless':
                self.assertIsInstance(conn.SpecificObject, NetworkManager.AccessPoint)
            if conn.Vpn:
                self.assertIsInstance(conn, NetworkManager.VPNConnection)
                self.assertIsInstance(conn.Banner, six.text_type)
                self.assertIsInstance(conn.SpecificObject, NetworkManager.ActiveConnection)
            self.assertTrue(conn.State == NetworkManager.NM_ACTIVE_CONNECTION_STATE_ACTIVATED)
            self.assertIsInstance(conn.Ip4Config, NetworkManager.IP4Config)
            self.assertIsInstance(conn.Ip6Config, (NetworkManager.IP6Config, type(None)))
            self.assertIsInstance(conn.Dhcp4Config, (NetworkManager.DHCP4Config, type(None)))
            self.assertIsInstance(conn.Dhcp6Config, (NetworkManager.DHCP6Config, type(None)))
            if conn.Master != None:
                self.assertIsInstance(conn.Master, NetworkManager.Device)
                self.assertEqual(conn.Master, conn.SpecificObject.Devices[0])

if __name__ == '__main__':
    unittest.main()



from test import *
import socket

class SettingsTest(TestCase):
    def test_connections(self):
        conn1 = NetworkManager.Settings.Connections
        conn2 = NetworkManager.Settings.ListConnections()
        self.assertIsInstance(conn1, list)
        for conn in conn1:
            self.assertIn(conn, conn2)
        for conn in conn2:
            self.assertIn(conn, conn1)
        conn = NetworkManager.Settings.GetConnectionByUuid(conn1[0].GetSettings()['connection']['uuid'])

    @unittest.skipUnless(os.getuid() == 0, "Must be root to reload connections")
    def test_reload(self):
        self.assertTrue(NetworkManager.Settings.ReloadConnections())

    @unittest.skipUnless(have_permission('settings.modify.hostname'), "don't have permission to modify the hostname")
    def test_hostname(self):
        hn = NetworkManager.Settings.Hostname
        self.assertEqual(hn, socket.gethostname())
        NetworkManager.Settings.SaveHostname(hn + '-test')
        self.assertEqual(NetworkManager.Settings.Hostname, hn + '-test')
        NetworkManager.Settings.SaveHostname(hn)
        self.assertEqual(NetworkManager.Settings.Hostname, hn)

if __name__ == '__main__':
    unittest.main()



from test import *

class ConnectionTest(TestCase):
    def test_settings(self):
        for connection in NetworkManager.Settings.ListConnections():
            settings = connection.GetSettings()
            self.assertIn(settings['connection']['type'], settings)

            secrets = connection.GetSecrets()
            for key in settings:
                self.assertIn(key, secrets)

            if 'ipv4' in settings:
                for address, prefix, gateway in settings['ipv4']['addresses']:
                    self.assertIsIpAddress(address)
                    self.assertIsIpAddress(gateway)
            if 'ipv6' in settings:
                for address, prefix, gateway in settings['ipv6']['addresses']:
                    self.assertIsIpAddress(address)
                    self.assertIsIpAddress(gateway)

    def test_update(self):
        active = [x.Connection for x in NetworkManager.NetworkManager.ActiveConnections]
        for connection in NetworkManager.Settings.Connections:
            if connection in active:
                continue
            settings = connection.GetSettings()
            connection.Update(settings)
            # FIXME: this causes assertion failures in n-m, which cause the dbus call to hang
            #settings['connection']['timestamp'] -= 1
            #connection.UpdateUnsaved(settings)
            #self.assertTrue(connection.Unsaved)
            #print("Saving")
            #connection.Save()
            #print("Saved")
            #self.assertFalse(connection.Unsaved)
            break

    def test_secrets(self):
        active = [x.Connection for x in NetworkManager.NetworkManager.ActiveConnections]
        key = '802-11-wireless-security' 
        for connection in NetworkManager.Settings.Connections:
            if connection in active:
                continue
            settings = connection.GetSettings()
            if key not in settings:
                continue
            secrets = connection.GetSecrets()
            if not secrets[key]:
                continue
            settings[key].update(secrets[key])

            connection.ClearSecrets()
            secrets = connection.GetSecrets()
            self.assertEqual(secrets[key], {})

            connection.Update(settings)
            secrets = connection.GetSecrets()
            self.assertNotEqual(secrets[key], {})
            break

if __name__ == '__main__':
    unittest.main()



from test import *

class AccessPointTest(TestCase):
    def test_accesspoints(self):
        for dev in NetworkManager.NetworkManager.Devices:
            if isinstance(dev, NetworkManager.Wireless):
                for ap in dev.AccessPoints:
                    self.assertIsInstance(ap.Flags, int)
                    # Frequencies from https://en.wikipedia.org/wiki/List_of_WLAN_channels
                    f = ap.Frequency
                    if not (
                        (f > 2400 and f < 2500) or
                        (f > 3650 and f < 3700) or
                        (f > 4900 and f < 6000)):
                        self.fail("Frequency is not a valid wifi frequency")
                    self.assertIsMacAddress(ap.HwAddress)
                    self.assertIsInstance(ap.LastSeen, int)
                    self.assertIsInstance(ap.MaxBitrate, int)
                    self.assertIsInstance(ap.WpaFlags, int)
                    self.assertIsInstance(ap.RsnFlags, int)
                    self.assertLess(ap.Strength, 100)
                    self.assertIsInstance(ap.Ssid, six.text_type)
                    self.assertIn(ap.Mode, (NetworkManager.NM_802_11_MODE_ADHOC, NetworkManager.NM_802_11_MODE_INFRA, NetworkManager.NM_802_11_MODE_AP))

if __name__ == '__main__':
    unittest.main()



"""
Listen to some available signals from NetworkManager
"""

import dbus.mainloop.glib
from gi.repository import GObject
import NetworkManager
import time

def out(msg):
    print("%s %s" % (time.strftime('%H:%M:%S'), msg))

def statechange(nm, interface, signal, state):
    out("State changed to %s" % NetworkManager.const('STATE', state))

def adddevice(nm, interface, signal, device_path):
    try:
        out("Device %s added" % device_path.IpInterface)
    except NetworkManager.ObjectVanished:
        # Sometimes this signal is sent for *removed* devices. Ignore.
        pass

def main():
    dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
    NetworkManager.NetworkManager.OnStateChanged(statechange)
    NetworkManager.NetworkManager.OnDeviceAdded(adddevice)

    out("Waiting for signals")
    loop = GObject.MainLoop()
    loop.run()

if __name__ == '__main__':
    main()



from test import *

class IpConfigTest(TestCase):
    def test_configs(self):
        for device in NetworkManager.NetworkManager.Devices:
            if device.State != NetworkManager.NM_DEVICE_STATE_ACTIVATED:
                continue
            self.do_check(device)
        for connection in NetworkManager.NetworkManager.ActiveConnections:
            self.do_check(connection)

    def do_check(self, thing):
        if thing.Dhcp4Config:
            self.assertIsInstance(thing.Dhcp4Config, NetworkManager.DHCP4Config)
            self.assertIsInstance(thing.Dhcp4Config.Options, dict)
            o = thing.Dhcp4Config.Options
            self.assertIsInstance(o['domain_name_servers'], list)
            self.assertIsInstance(o['ntp_servers'], list)
            self.assertIsIpAddress(o['ip_address'])
            for key in o:
                if key.endswith('_requested'):
                    self.assertTrue(o[key])
        if thing.Dhcp6Config:
            self.assertIsInstance(thing.Dhcp6Config, NetworkManager.DHCP6Config)
            self.assertIsInstance(thing.Dhcp6Config.Options, dict)
        for c in (thing.Ip4Config, thing.Ip6Config):
            if not c:
                continue
            for addr, prefix, gateway in c.Addresses:
                self.assertIsIpAddress(addr)
                self.assertIsIpAddress(gateway)
                self.assertIsIpNetwork(addr, prefix)
            for data in c.AddressData:
                self.assertIsIpAddress(data['address'])
                self.assertIsIpNetwork(data['address'], data['prefix'])
                if 'peer' in data:
                    self.assertIsIpAddress(data['peer'])
            if c.Gateway:
                self.assertIsIpAddress(c.Gateway)
            for addr in c.Nameservers:
                self.assertIsIpAddress(addr)
            for addr in getattr(c, 'WinsServers', []):
                self.assertIsIpAddress(addr)
            for dest, prefix, next_hop, metric in c.Routes:
                self.assertIsIpNetwork(dest, prefix)
                self.assertIsIpAddress(next_hop)
                self.assertLessEqual(metric, 1000)
            for data in c.RouteData:
                self.assertIsIpNetwork(data['dest'], data['prefix'])
                self.assertIsIpAddress(data['next-hop'])
                self.assertLessEqual(data['metric'], 1000)

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

setup(name = "python-networkmanager",
      version = "2.0.1",
      author = "Dennis Kaarsemaker",
      author_email = "dennis@kaarsemaker.net",
      url = "http://github.com/seveas/python-networkmanager",
      description = "Easy communication with NetworkManager",
      py_modules = ["NetworkManager"],
      install_requires = ["dbus-python", "six"],
      classifiers = [
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: zlib/libpng License',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Topic :: System :: Networking',
      ]
)
